import { dev } from "$app/environment";
import { API_URL } from "$env/static/private";

import type { Actions, PageServerLoad } from "./$types";

import { redirect } from "@sveltejs/kit";

import { fail, setError, superValidate } from "sveltekit-superforms";
import { zod } from 'sveltekit-superforms/adapters';

import { COOKIES } from "$lib/constants";

import { formConfirmationSchema, formSchema } from "./schema";

export const load: PageServerLoad = async ({ locals }) => {
  if (locals.user) {
    redirect(302, "/");
  }

  return {
    title: 'Inscription',

    form: await superValidate(zod(formSchema)),
    formConfirmation: await superValidate(zod(formConfirmationSchema)),
  }
};

export const actions: Actions = {
  register: async ({ request, locals, fetch }) => {
    if (locals.user) {
      redirect(302, "/");
    }

    const form = await superValidate(request, zod(formSchema));

    if (!form.valid) {
      return fail(400, {
        form,
      });
    }

    try {
      const res = await fetch(API_URL + "/register", {
        method: "POST",
        body: JSON.stringify({ email: form.data.email }),
      });

      if (!res.ok) {
        if (res.status === 400) {
          const { email_valid, username_valid } = await res.json();
          if (!email_valid) return setError(form, 'email', 'Un compte avec cette adresse email existe déjà');
          if (!username_valid) return setError(form, 'pseudo', "Ce nom d'utilisateur est déjà utilisé");
        }

        return setError(form, 'email', "Une erreur est survenue, veuillez réessayer");
      }

      return {
        form
      }
    } catch (error) {
      console.error(error);
      return setError(form, 'pseudo', "Une erreur est survenue, veuillez réessayer");
    }
  },
  confirmation: async ({ request, locals, fetch, cookies, url }) => {
    if (locals.user) {
      redirect(302, "/");
    }

    const form = await superValidate(request, zod(formConfirmationSchema));

    if (!form.valid) {
      return fail(400, {
        form,
      });
    }

    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    const { confirm, ...body } = form.data;

    try {
      const res = await fetch(API_URL + "/confirmation", {
        method: "POST",
        body: JSON.stringify({ ...body }),
      });

      if (!res.ok) {
        if (res.status === 400) {
          const { email_valid, username_valid } = await res.json();
          if (!email_valid) return setError(form, 'email', 'Un compte avec cette adresse email existe déjà');
          if (!username_valid) return setError(form, 'pseudo', "Ce nom d'utilisateur est déjà utilisé");
        }

        return setError(form, 'email', "Une erreur est survenue, veuillez réessayer");
      }

      const token = res.headers.get("Authorization")?.split("Bearer ").pop();

      if (!token) {
        return setError(form, 'code', "Une erreur est survenue, veuillez réessayer");
      }

      cookies.set(COOKIES.SESSION, token, {
        path: "/",
        secure: !dev,
        sameSite: "strict",
      });
    } catch (error) {
      console.error(error);
      return setError(form, 'code', "Une erreur est survenue, veuillez réessayer");
    }

    const redirectTo = url.searchParams.get('redirectTo');

    if (redirectTo) {
      redirect(302, `/${redirectTo.slice(1)}`);
    }

    redirect(302, "/");
  },
};
